const bitSensorServer = require('../core/bitsensor_server');
const cls = require('continuation-local-storage');
const chai = require('chai');
const chaiHttp = require('chai-http');

const { expect } = chai;
const timeout = 50;

chai.use(chaiHttp);
process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';

describe('End to end test', () => {
  after('Close servers', (done) => {
    bitSensorServer.server.close();
    done();
  });
  beforeEach(() => {
    bitSensorServer.dataCollector.length = 0;
  });
  let clientServer;
  describe('app.listen monkey patch', () => {
    before(() => {
      clientServer = require('../core/client_server')
    });

    after(() => {
      clientServer.server.close();
      cls.destroyNamespace('bitsensor');
      cls.createNamespace('bitsensor');
    });

    describe('succesful test', () => {
      it('handling request should be done', (done) => {
        chai.request('http://localhost:3003')
          .get('/')
          .end(() => {
            expect(bitSensor.getHandlingRequest()).to.be.undefined;
            done();
          });
      });
      it('response should be approved', (done) => {
        chai.request('http://localhost:3003')
          .get('/')
          .end((err, res) => {
            expect(res.res.socket._httpMessage.res.text).to.equal('approved');
            done();
          });
      });
      it('should send at least one datapoint', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3003')
            .get('/')
            .end();
          setTimeout(() => {
            const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
            expect(data).to.be.an('object');
            expect(data.context).to.have.property('ip');
            expect(data.endpoint).to.have.property('port', '3003');
            expect(data.meta).to.have.property('provider', 'bitsensor-nodejs');
            done();
          }, timeout);
        });
      });
    });
    describe('error test', () => {
      it('response should be error', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3003')
            .get('/err')
            .end((err, res) => {
              expect(res.res.socket._httpMessage.res.text).to.not.equal(200);
              done();
            });
        });
      });
      it('should contain error', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3003')
            .get('/err')
            .end();
        });
        setTimeout(() => {
          const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
          expect(data).to.be.an('object');
          expect(data.errors).has.lengthOf(1);
          done();
        }, timeout);
      });
    });
    describe('httphandlerManager middleware', () => {
      it('should not be undefined', (done) => {
        expect(httpHandlerManager).to.not.be.undefined;
        done();
      });
    });
  });

  describe('http server monkey patch end to end test', () => {
    before(() => {
      clientServer = require('../core/http_server');
    });

    after(() => {
      clientServer.server.close();
      cls.destroyNamespace('bitsensor');
      cls.createNamespace('bitsensor');
    });

    describe('succesful test', () => {
      it('handling request should be done', (done) => {
        chai.request('http://localhost:3004')
          .get('/')
          .end((err, res) => {
            expect(bitSensor.getHandlingRequest()).to.be.undefined;
            done();
          });
      });
      it('response should be approved', (done) => {
        chai.request('http://localhost:3004')
          .get('/')
          .end((err, res) => {
            expect(res.res.socket._httpMessage.res.text).to.equal('approved');
            done();
          });
      });
      it('should send at least one datapoint', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3004')
            .get('/')
            .end();
        });
        setTimeout(() => {
          const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
          expect(data).to.be.an('object');
          expect(data.context).to.have.property('ip');
          expect(data.endpoint).to.have.property('port', '3004');
          expect(data.meta).to.have.property('provider', 'bitsensor-nodejs');
          done();
        }, timeout);
      });
    });
    describe('error test', () => {
      it('response should be error', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3004')
            .get('/err')
            .end((err, res) => {
              expect(res.res.socket._httpMessage.res.text).to.not.equal(200);
              done();
            });
        });
      });
      it('should contain error', (done) => {
        bitSensor.runSession(() => {
          chai.request('http://localhost:3004')
            .get('/err')
            .end();
        });
        setTimeout(() => {
          const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
          expect(data).to.be.an('object');
          expect(data.errors).to.have.lengthOf(1);
          done();
        }, timeout);
      });
    });
  });
  // HTTPS
  describe('https server monkey patch end to end test', () => {
    before(() => {
      clientServer = require('../core/https_server');
    });

    after(() => {
      clientServer.server.close();
      cls.destroyNamespace('bitsensor');
      cls.createNamespace('bitsensor');
    });

    describe('succesful test', () => {
      it('handling request should be done', (done) => {
        chai.request('https://localhost:8443')
          .get('/')
          .end((err, res) => {
            expect(bitSensor.getHandlingRequest()).to.be.undefined;
            done();
          });
      });
      it('response should be approved', (done) => {
        chai.request('https://localhost:8443')
          .get('/')
          .end((err, res) => {
            expect(res.res.socket._httpMessage.res.text).to.equal('approved');
            done();
          });
      });
      it('should send at least one datapoint', (done) => {
        bitSensor.runSession(() => {
          chai.request('https://127.0.0.1:8443')
            .get('/')
            .end();
        });
        setTimeout(() => {
          const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
          expect(data).to.be.an('object');
          expect(data.context).to.have.property('ip');
          expect(data.endpoint).to.have.property('port', '8443');
          expect(data.meta).to.have.property('provider', 'bitsensor-nodejs');
          done();
        }, timeout);
      });
    });
    describe('error test', () => {
      it('response should be error', (done) => {
        bitSensor.runSession(() => {
          chai.request('https://localhost:8443')
            .get('/err')
            .end((err, res) => {
              expect(res.res.socket._httpMessage.res.text).to.not.equal(200);
              done();
            });
        });
      });
      it('should contain error', (done) => {
        bitSensor.runSession(() => {
          chai.request('https://localhost:8443')
            .get('/err')
            .end();
        });
        setTimeout(() => {
          const data = JSON.parse(bitSensorServer.dataCollector[bitSensorServer.dataCollector.length - 1].data);
          expect(data).to.be.an('object');
          expect(data.errors).to.have.lengthOf(1);
          done();
        }, timeout);
      });
    });
  });
});
